module vga_controller_stream ( 

// avalon streaming slave interface

	address,
	clk,
	reset,
	cs,
	write,
	fifo_data,
	readyfordata,
	endofpacket,

// vga controller interface

	hsync,
	vsync,
	sync,
	blank,
	rgb
);

input address;
input clk;
input reset;
input cs;
input write;
input [7:0] fifo_data;
output readyfordata;
output endofpacket;

output hsync;
output vsync;
output sync;
output blank;
output [2:0] rgb;


wire readyfordata;
wire endofpacket;

wire hsync;
wire vsync;
wire sync;
wire blank;
wire [7:0] RGB8;
wire [7:0] pix_val;
wire read_next_pixel;
wire end_of_picture;
wire [9:0] fifo_cnt;

reg [19:0] pixel_count;
reg sync_end_of_picture;
reg vgaclk;
reg vgaclk1;
// readyfordata is used to pace arriving pixels into the FIFO
assign readyfordata = ((fifo_cnt < 10'd1000) && (pixel_count < (640*480)) && (!sync_end_of_picture)) ? 1'b1 : 1'b0;
// endofpacket signals the DMA controller that the transfer is complete
assign endofpacket = ((pixel_count > ((640*480)-2)) && cs && write) ? 1'b1 : 1'b0;

// pixel_count counts the number of pixels written into the FIFO
// so we can stop and signal end-of-packet once we have all the pixels
// this block is also where we synchronize the pixel dma with the start of a new video frame
always @ (posedge clk or posedge reset)
begin
	if ( reset == 1'b1 )
	begin
		pixel_count <= 20'h000;
		sync_end_of_picture <= 1'b0;
	end
	else
	begin
		sync_end_of_picture <= end_of_picture;
// reset on end_of_picture only if we've read all pixels, 

		if ( sync_end_of_picture && (pixel_count > ((640*480)-1)) )	

// this provides sync with dma and start of new frame
		begin													
			pixel_count <= 20'h000;								
		end
		else
		begin
			if ( cs && write && readyfordata )
			begin
				pixel_count <= pixel_count + 1'b1;
			end
		end
	end
end

always @(posedge clk) begin
  vgaclk1<=~vgaclk1;
 end
always @(posedge vgaclk1) begin
  vgaclk<=~vgaclk;
 end
assign rgb=RGB8[2:0];
// this module generates all the video timing
vga_timing vga_timing_inst ( 
	.reset ( reset ),
	.vga_clk ( vgaclk ),
	.hsync ( hsync ),
	.vsync ( vsync ),
	.sync ( sync ),
	.blank ( blank ),
	.RGB ( RGB8 ),
	.pix_val ( pix_val ),
	.read_next_pixel ( read_next_pixel ),
	.end_of_picture ( end_of_picture )
);

// this module provides a 1024 byte pixel FIFO which is just over a line and a half
vga_pixel_fifo	vga_pixel_fifo_inst (
	.data ( fifo_data ),
	.wrreq ( cs & write & readyfordata ),
	.rdreq ( read_next_pixel ),
	.rdclk ( vgaclk ),
	.wrclk ( clk ),
	.aclr ( sync_end_of_picture ),
	.q ( pix_val ),
	.wrusedw ( fifo_cnt )
	);

endmodule
