/************************************************
* www.21control.com
 ************************************************/
#ifdef LWIP

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <altera_avalon_pio_regs.h>

#include "alt_types.h"
#include "ctrl_rtl8019_regs.h"
#include "lwip/opt.h"
#include "lwip/def.h"
#include "lwip/mem.h"
#include "lwip/pbuf.h"
#include "lwip/sys.h"
#include "lwip/netif.h"
#include "lwip/stats.h"
#include "netif/etharp.h"
#include "system.h"
#include "arch/perf.h"
#include "sys/alt_irq.h"
#include "ctrl_rtl8019.h"


/* Unique descricption of the network interface. */
#define IFNAME0 'n'
#define IFNAME1 '9'

extern sys_mbox_t  rx_mbox;

static err_t
ctrl_rtl8019_output(struct netif *netif, struct pbuf *p,struct ip_addr *ipaddr);
static struct pbuf* low_level_input(ctrl_rtl8019_if *dev);
static err_t low_level_output(struct netif *netif, struct pbuf *p);
static err_t low_level_init(ctrl_rtl8019_if* dev);
void ctrl_rtl8019_irq(void* context, alt_u32 interrupt);

/*************************************************/

static unsigned char mac[]={0x01,0x02,0x03,0x04,0x05,0x06};

static inline void Delay16Cycles(void)
{
    unsigned int i;
    for(i=0;i<100;i++);
}

u_char reg_read(unsigned int base,u_char reg)
{
  unsigned char uc;
  
  uc=IORD_8DIRECT(base, reg); /* DYNAMIC MEMORY */
  //uc=IORD(base, reg);       /* NATIVE MEMORY SIZE */
  return uc;  
} 

u_char reg_write(base,reg, data)
{ 
  IOWR_8DIRECT(base, reg, data);/* DYNAMIC MEMORY */
  //IOWR(base, reg, data);      /* NATIVE MEMORY SIZE */
  return data;
}

/*
void getState(void)
{
  unsigned char old_cr,uc;
  unsigned char tsr,isr,rsr;
  unsigned char rcr,tcr,dcr,imr;
  unsigned char  pstart,pstop;
  int iRet;
  old_cr=IORD_CTRL_RTL8019_CR(RTL_BASE_ADDR);
  
  //change to page0 
  IOWR_CTRL_RTL8019_CR(RTL_BASE_ADDR,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK );
  tsr=IORD_CTRL_RTL8019_TSR(RTL_BASE_ADDR);
  isr=IORD_CTRL_RTL8019_ISR(RTL_BASE_ADDR);
  rsr=IORD_CTRL_RTL8019_RSR(RTL_BASE_ADDR);
  
  // change to page2 
  IOWR_CTRL_RTL8019_CR(RTL_BASE_ADDR, CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK |  CTRL_RTL8019_CR_PS1_MSK);
  
  pstop=IORD_CTRL_RTL8019_PSTOP(RTL_BASE_ADDR);
  pstart=IORD_CTRL_RTL8019_PSTART(RTL_BASE_ADDR);
  rcr=IORD_CTRL_RTL8019_RCR(RTL_BASE_ADDR);
  tcr=IORD_CTRL_RTL8019_TCR(RTL_BASE_ADDR);
  dcr=IORD_CTRL_RTL8019_DCR(RTL_BASE_ADDR);
  imr=IORD_CTRL_RTL8019_IMR(RTL_BASE_ADDR);
  
  IOWR_CTRL_RTL8019_CR(RTL_BASE_ADDR,old_cr); 
}
*/

/*!
 * Reset the Ethernet controller.
 * use hardware line e_rst
 */
int rtl8019_Reset(ctrl_rtl8019_if* dev)
{
    unsigned char  i;
    unsigned char  j;
    
    /*
     * Do the software reset by reading from the reset register followed
     * by writing to the reset register. Wait until the controller enters
     * the reset state.
     */
    for (j = 0; j < 20; j++) {
        IOWR_ALTERA_AVALON_PIO_DATA(E_RST_BASE,0x01);
        OSTimeDlyHMSM(0,0,1,0);
        IOWR_ALTERA_AVALON_PIO_DATA(E_RST_BASE,0x00);
        for (i = 0; i < 20; i++) {
            OSTimeDlyHMSM(0,0,1,0);
            /*
             * We got the reset bit. However, Ethernut 1.1 may
             * still fail because the NIC hasn't got it's hardware
             * reset and the data lines remain in tristate. So we
             * read noise instead of the register. To solve this
             * problem, we will verify the NIC's id.
             */
            i=IORD_CTRL_RTL8019_ISR(dev->base_addr);
            if ((i & CTRL_RTL8019_ISR_RST_MSK ) != 0 )
                return 0;
                  
            i=IORD_CTRL_RTL8019_8019ID0(dev->base_addr); 
            if ( i == 0x50 )
                return 0;

            i= IORD_CTRL_RTL8019_8019ID1(dev->base_addr);
            if( i == 0x70)
                return 0; 
        }
    }
    return -1;
}


/*-----------------------------------------------------------------------------------* 
 *
 * ctrl_rtl8019_input():
 *
 * This function should be called when a packet is ready to be read
 * from the interface. It uses the function low_level_input() that
 * should handle the actual reception of bytes from the network
 * interface.
 *
 * 
 *-----------------------------------------------------------------------------------*/
static void ctrl_rtl8019_input(ctrl_rtl8019_if *dev)
{
  struct eth_hdr  *ethhdr;
  struct pbuf     *p;
  struct netif*   netif = dev->lwip_dev_list.dev.netif;

  /* move received packet into a new pbuf */
  p = low_level_input(dev);

  /* no packet could be read, silently ignore this */
  if (p == NULL) return;

  ethhdr = p->payload;

  switch (htons(ethhdr->type)) {
  /* IP packet? */
  case ETHTYPE_IP:
    /* 
    * update ARP table 
    * 
    * The code which updates the ARP tables does not appear to be thread safe
    * so I've added a MUTEX around all calls to the arp code
    * 
    */
    sys_sem_wait(dev->arp_semaphore);
    etharp_ip_input(netif, p);
    sys_sem_signal(dev->arp_semaphore);
    /* skip Ethernet header */
    pbuf_header(p, -(s16_t)sizeof(struct eth_hdr));
    /* pass to network layer */
    netif->input(p, netif);
    break;
      
    case ETHTYPE_ARP:
      /* 
      * pass p to ARP module  
      * 
      * The code which updates the ARP tables does not appear to be thread safe
      * so I've added a MUTEX around all calls to the arp code
      * 
      */
      sys_sem_wait(dev->arp_semaphore);
      etharp_arp_input(netif, (struct eth_addr *)&netif->hwaddr, p);
      sys_sem_signal(dev->arp_semaphore);
      break;
    default:
      pbuf_free(p);
      p = NULL;
      break;
  }
}

/*-----------------------------------------------------------------------------------*
 *
 * ctrl_rtl8019_init():
 *
 * Should be called at the beginning of the program to set up the
 * network interface. It calls the function low_level_init() to do the
 * actual setup of the hardware.
 *
 * 
 *-----------------------------------------------------------------------------------*/
err_t ctrl_rtl8019_init(struct netif *netif)
{
  err_t ret_code;
  ctrl_rtl8019_if* dev = (ctrl_rtl8019_if*)netif->state;
  dev->lwip_dev_list.dev.netif = netif;

  LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("ctrl_rtl8019_init()\n"));
  netif->name[0] = IFNAME0;
  netif->name[1] = IFNAME1;
  netif->output = ctrl_rtl8019_output;
  netif->linkoutput = low_level_output;

  if (netif->state == NULL ) 
  {
    ret_code = ERR_IF;
    goto exit;
  }

  if (dev->bus_width != 8)
  {
    printf("This driver doesn't support the Ethernet add on card at present\n");
    ret_code = ERR_IF;
    goto exit;
  }

  ret_code = low_level_init(dev);
  if (ret_code == ERR_OK)
  {
    etharp_init();
  }
  else
  {
    free(netif->state);
  }

exit:
  LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("ctrl_rtl8019_init() exit = %d\n",ret_code));

  return ret_code;
}

/*
 * Fires up the network interface. NIC interrupts
 * should have been disabled when calling this
 * function.
 */
static err_t low_level_init(ctrl_rtl8019_if* dev)
{
    unsigned char i;

  /* 
  * Need to have a pointer to a generice lwip_dev
  * This is so that the get_mac_addr function interface
  * is device agnostic
  */
    alt_lwip_dev* lwip_dev = &dev->lwip_dev_list.dev;
    err_t ret_code = ERR_OK;
    
    LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("low_level_init()\n"));
    
  
  /* Set MAC hardware address length */
  lwip_dev->netif->hwaddr_len = 6;
  
    /* reset chip */
    if (rtl8019_Reset(dev)) {
        return -1;
    }
    
    /*
     * Mask all interrupts and clear any interrupt status flag to set the
     * INT pin back to low.
     */
    IOWR_CTRL_RTL8019_IMR(dev->base_addr,0x00);
    IOWR_CTRL_RTL8019_ISR(dev->base_addr,0xff);
    OSTimeDlyHMSM(0,0,1,0);
    /*
     * During reset the nic loaded its initial configuration from an
     * external eeprom. On the ethernut board we do not have any
     * configuration eeprom, but simply tied the eeprom data line to
     * high level. So we have to clear some bits in the configuration
     * register. Switch to register page 3.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STP_MSK | CTRL_RTL8019_CR_RD2_MSK | CTRL_RTL8019_CR_PS0_MSK | CTRL_RTL8019_CR_PS1_MSK);
    
    /*
     * The nic configuration registers are write protected unless both
     * EEM bits are set to 1.
     */
    IOWR_CTRL_RTL8019_9346CR(dev->base_addr, CTRL_RTL8019_9346CR_EEM0_MSK | CTRL_RTL8019_9346CR_EEM1_MSK);
    
    /*
     * Network media had been set to 10Base2 by the virtual EEPROM and
     * will be set now to auto detect. This will initiate a link test.
     * We don't force 10BaseT, because this would disable the link test.
     */
    IOWR_CTRL_RTL8019_CONFIG2(dev->base_addr, CTRL_RTL8019_CONFIG2_BSELB_MSK);
    
    /*
     * Disable sleep and power down.
     *
     * The virtual EEPROM (resistor tight to VCC) will set all bits of
     * CONFIG3 to 1. Unfortunately we are not able to modify the full
     * duplex bit. The only solution is to use a real EEPROM or emulate
     * one.
     */
    //reg_write(RTL_BASE_ADDR,NIC_PG3_CONFIG3, NIC_CONFIG3_LEDS0 | NIC_CONFIG3_LEDS1);
    IOWR_CTRL_RTL8019_CONFIG3(dev->base_addr,CTRL_RTL8019_CONFIG3_LEDS0_MSK|CTRL_RTL8019_CONFIG3_LEDS1_MSK );
    
    /*
     * Reenable write protection of the nic configuration registers
     * and wait for link test to complete.
     */
    IOWR_CTRL_RTL8019_9346CR(dev->base_addr, 0);
    
    OSTimeDlyHMSM(0,0,1,0);
    
    /*
     * Switch to register page 0 and set data configuration register
     * to byte-wide DMA transfers, normal operation (no loopback),
     * send command not executed and 8 byte fifo threshold.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STP_MSK | CTRL_RTL8019_CR_RD2_MSK);
    IOWR_CTRL_RTL8019_DCR(dev->base_addr,CTRL_RTL8019_DCR_LS_MSK | CTRL_RTL8019_DCR_FT1_MSK);

    /*
     * Clear remote dma byte count register.
     */
    IOWR_CTRL_RTL8019_RBCR0(dev->base_addr,0);
    IOWR_CTRL_RTL8019_RBCR1(dev->base_addr,0);

    /*
     * Temporarily set receiver to monitor mode and transmitter to
     * internal loopback mode. Incoming packets will not be stored
     * in the nic ring buffer and no data will be send to the network.
     */
    IOWR_CTRL_RTL8019_RCR(dev->base_addr,CTRL_RTL8019_RCR_MON_MSK);
    IOWR_CTRL_RTL8019_TCR(dev->base_addr,CTRL_RTL8019_TCR_LB0_MSK);

    /*
     * Configure the nic's ring buffer page layout.
     * NIC_PG0_BNRY: Last page read.
     * NIC_PG0_PSTART: First page of receiver buffer.
     * NIC_PG0_PSTOP: Last page of receiver buffer.
     */
    IOWR_CTRL_RTL8019_TPSR(dev->base_addr,NIC_FIRST_TX_PAGE);
    IOWR_CTRL_RTL8019_BNRY(dev->base_addr,NIC_STOP_PAGE - 1);
    IOWR_CTRL_RTL8019_PSTART(dev->base_addr,NIC_FIRST_RX_PAGE);
    IOWR_CTRL_RTL8019_PSTOP(dev->base_addr, NIC_STOP_PAGE);
    
    /*
     * Once again clear interrupt status register.
     */
    IOWR_CTRL_RTL8019_ISR(dev->base_addr,0xff);
    
    /*
     * Switch to register page 1 and copy our MAC address into the nic.
     * We are still in stop mode.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STP_MSK | CTRL_RTL8019_CR_RD2_MSK | CTRL_RTL8019_CR_PS0_MSK);
    for (i = 0; i < 6; i++)
        reg_write(dev->base_addr,1 + i, mac[i]);

    /*
     * Clear multicast filter bits to disable all packets.
     */
    for (i = 0; i < 8; i++)
        reg_write(dev->base_addr,8 + i, 0);

    /*
     * Set current page pointer to one page after the boundary pointer.
     */
    IOWR_CTRL_RTL8019_CURR(dev->base_addr,NIC_FIRST_RX_PAGE);
   
    /*
     * Switch back to register page 0, remaining in stop mode.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STP_MSK | CTRL_RTL8019_CR_RD2_MSK);

    /*
     * Take receiver out of monitor mode and enable it for accepting
     * broadcasts.
     */
    IOWR_CTRL_RTL8019_RCR(dev->base_addr, CTRL_RTL8019_RCR_AB_MSK);
    
    /*
     * Clear all interrupt status flags and enable interrupts.
     */
    IOWR_CTRL_RTL8019_ISR(dev->base_addr,0xff);
    
    /* Note: transmitter if polled, thus no NIC_IMR_PTXE */
    IOWR_CTRL_RTL8019_IMR(dev->base_addr,CTRL_RTL8019_IMR_PRXE_MSK | CTRL_RTL8019_IMR_RXEE_MSK | CTRL_RTL8019_IMR_TXEE_MSK | CTRL_RTL8019_IMR_OVWE_MSK);
    
    /*
     * Fire up the nic by clearing the stop bit and setting the start bit.
     * To activate the local receive dma we must also take the nic out of
     * the local loopback mode.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK);
    IOWR_CTRL_RTL8019_TCR(dev->base_addr,0);
    OSTimeDlyHMSM(0,0,1,0);
    /* maximum transfer unit */
    dev->lwip_dev_list.dev.netif->mtu = 1500;
  
    /* broadcast capability */
    dev->lwip_dev_list.dev.netif->flags = NETIF_FLAG_BROADCAST;
  
    dev->semaphore = sys_sem_new(1);
    if (dev->semaphore == NULL)
    {
        ret_code = ERR_MEM;
        goto exit;
    }

    dev->arp_semaphore = sys_sem_new(1);
    if (dev->arp_semaphore == NULL)
    {
        ret_code = ERR_MEM;
        goto exit;
    }
    
    if (alt_irq_register ( dev->irq, dev, 
                          ctrl_rtl8019_irq))
    {
        ret_code = ERR_IF;
        goto exit;
    }
    
    exit:
    LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("low_level_init() exit = %d\n", ret_code));

    return ret_code;
}

/*!
 * Complete remote DMA.
 */
static void NicCompleteDma(unsigned int addr)
{
    u_char i;

    /*
     * Complete remote dma.
     */
    IOWR_CTRL_RTL8019_CR(addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK);

    /*
     * Check that we have a DMA complete flag.
     */
    for (i = 0; i <= 20; i++)
        if (IORD_CTRL_RTL8019_ISR(addr) & CTRL_RTL8019_ISR_RDC_MSK)
            break;

    /*
     * Reset remote dma complete flag.
     */
    IOWR_CTRL_RTL8019_ISR(addr,CTRL_RTL8019_ISR_RDC_MSK);
}

/*
 * Write data block to the NIC.
 */
static void NicWrite(unsigned int addr,u_char * buf, u_short len)
{
    register u_short l = len - 1;
    register u_char ih = (u_short) l >> 8;
    register u_char il = (u_char) l;

    if (!len)
        return;

    do {
        do {
            IOWR_CTRL_RTL8019_DATA(addr,*buf++);
        } while (il-- != 0);
    } while (ih-- != 0);
}

/*
 * Read data block from the NIC.
 */
static void NicRead(unsigned int addr,u_char * buf, u_short len)
{
    register u_short l = len - 1;
    register u_char ih = (u_short) l >> 8;
    register u_char il = (u_char) l;

    if (!len)
        return;

    do {
        do {
            *buf++ = IORD_CTRL_RTL8019_DATA(addr);
        } while (il-- != 0);
    } while (ih-- != 0);
}


/*
* ctrl_rtl8019_rx
* Handle all the receives
*/
void ctrl_rtl8019_rx(  alt_lwip_dev* dev )
{
    alt_u8  isr;
    ctrl_rtl8019_if *lan_dev = (ctrl_rtl8019_if *)dev->netif->state;
  
    LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("rtl8019_rx()"));

    IOWR_CTRL_RTL8019_CR(lan_dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK);
    isr=IORD_CTRL_RTL8019_ISR(lan_dev->base_addr);
    while(isr & CTRL_RTL8019_ISR_PRX_MSK)
    {
        ctrl_rtl8019_input(lan_dev);
        IOWR_CTRL_RTL8019_CR(lan_dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK);
    	isr=IORD_CTRL_RTL8019_ISR(lan_dev->base_addr);
    }
  
    /* Re-enable RX interrupts */
    IOWR_CTRL_RTL8019_IMR(lan_dev->base_addr,CTRL_RTL8019_IMR_PRXE_MSK | CTRL_RTL8019_IMR_RXEE_MSK | CTRL_RTL8019_IMR_TXEE_MSK | CTRL_RTL8019_IMR_OVWE_MSK);
}


/*!
 * \brief Fetch the next packet out of the receive ring buffer.
 *
 * Nic interrupts must be disabled when calling this funtion.
 */
/*-----------------------------------------------------------------------------------* 
 *
 * low_level_input():
 *
 * Should allocate a pbuf and transfer the bytes of the incoming
 * packet from the interface into the pbuf.
 *
 *
 *-----------------------------------------------------------------------------------*/
static struct pbuf*
low_level_input(ctrl_rtl8019_if *dev)
{
    struct pbuf *p=NULL, *q;
  
    struct nic_pkt_header hdr;
    u_short count;
    u_char *buf;
    u_char nextpg;
    u_char bnry;
    u_char curr;
    u_short i;
    u_char drop = 0;
    
    LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("low_level_input()\n"));
    
    /* we don't want to be interrupted by NIC owerflow */
    //irq_context=alt_irq_disable_all();
    
    /*
     * Get the current page pointer. It points to the page where the NIC
     * will start saving the next incoming packet.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr, CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK | CTRL_RTL8019_CR_PS0_MSK);
    Delay16Cycles();
    curr = IORD_CTRL_RTL8019_CURR(dev->base_addr);
    IOWR_CTRL_RTL8019_CR(dev->base_addr, CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK);

    /*
     * Get the pointer to the last page we read from. The following page
     * is the one where we start reading. If it's equal to the current
     * page pointer, then there's nothing to read. In this case we return
     * a null pointer.
     */
    if ((bnry = IORD_CTRL_RTL8019_BNRY(dev->base_addr) + 1) >= NIC_STOP_PAGE)
        bnry = NIC_FIRST_RX_PAGE;

    if (bnry == curr) {
        //alt_irq_enable_all(irq_context);
        return 0;
    }

    /*
     * Read the NIC specific packet header.
     */
    IOWR_CTRL_RTL8019_RBCR0(dev->base_addr,4);
    IOWR_CTRL_RTL8019_RBCR1(dev->base_addr,0);
    IOWR_CTRL_RTL8019_RSAR0(dev->base_addr,0);
    IOWR_CTRL_RTL8019_RSAR1(dev->base_addr,bnry);
    buf = (u_char *) & hdr;
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD0_MSK);
    Delay16Cycles();
    for (i = 0; i < sizeof(struct nic_pkt_header); i++)
    {
        *buf=IORD_CTRL_RTL8019_DATA(dev->base_addr);
        buf++;
    }
    NicCompleteDma(dev->base_addr);
    
    /*
     *  Check packet length. Silently discard packets of illegal size.
     */
    if (hdr.ph_size < 60 + sizeof(struct nic_pkt_header) ||     /* */
        hdr.ph_size > 1514 + sizeof(struct nic_pkt_header)) {
        drop = 1;
    }

    /*
     * Calculate the page of the next packet. If it differs from the
     * pointer in the packet header, we return with errorcode.
     */
    nextpg = bnry + (hdr.ph_size >> 8) + ((hdr.ph_size & 0xFF) != 0);
    if (nextpg >= NIC_STOP_PAGE) {
        nextpg -= NIC_STOP_PAGE;
        nextpg += NIC_FIRST_RX_PAGE;
    }
    if (nextpg != hdr.ph_nextpg) {
        u_char nextpg1 = nextpg + 1;
        if (nextpg1 >= NIC_STOP_PAGE) {
            nextpg1 -= NIC_STOP_PAGE;
            nextpg1 += NIC_FIRST_RX_PAGE;
        }
        if (nextpg1 != hdr.ph_nextpg) {
            return NULL;
        }
        nextpg = nextpg1;
    }
    
    
    /*
     * Check packet status. It should have set bit 0, but
     * even without this bit packets seem to be OK.
     */
    if (!drop && ((hdr.ph_status & 0x0E) == 0)) {
        /*
         * Allocate a NETBUF.
         * Omit the fcs.
         */
        count = hdr.ph_size - 4;
        /* We allocate a pbuf chain of pbufs from the pool. */
    	p = pbuf_alloc(PBUF_RAW, count, PBUF_POOL);
        if (p != NULL ) {
            /*
             * Set remote dma byte count and
             * start address. Don't read the
             * header again.
             */
            IOWR_CTRL_RTL8019_RBCR0(dev->base_addr,count);
            IOWR_CTRL_RTL8019_RBCR1(dev->base_addr,count >> 8);
            IOWR_CTRL_RTL8019_RSAR0(dev->base_addr,sizeof(struct nic_pkt_header));
            IOWR_CTRL_RTL8019_RSAR1(dev->base_addr,bnry);

            /*
             * Perform the read.
             */
            IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD0_MSK);
            Delay16Cycles();
            for(q = p; q != NULL; q = q->next) 
      	    {
            	NicRead(dev->base_addr,q->payload, q->len);
            }
            NicCompleteDma(dev->base_addr);
        }
    }

    /*
     * Set boundary register to the last page we read.
     * This also drops packets with errors
     */
    if (--nextpg < NIC_FIRST_RX_PAGE)
        nextpg = NIC_STOP_PAGE - 1;
    IOWR_CTRL_RTL8019_BNRY(dev->base_addr,nextpg);
      
    return p;
}

/*!
 * Load a packet into the nic's transmit ring buffer.
 *
 *
 * param base NIC hardware base address.
 * param nb Network buffer structure containing the packet to be sent.
 *           The structure must have been allocated by a previous
 *           call NutNetBufAlloc().
 *
 * return 0 on success, -1 in case of any errors. Errors
 *         will automatically release the network buffer
 *         structure.
 */
static err_t
low_level_output(struct netif *netif, struct pbuf *p)
{
    struct pbuf *q;
    ctrl_rtl8019_if *dev = netif->state;
    u_short i;
    u_char padding = 0;
    unsigned short int size;
    

    LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("low_level_output ( %#x)\n",p));
    
    size=0;
    for(q = p; q != NULL; q = q->next) 
    {
        size+=q->len;
    }
    
    /*
     * Calculate the number of bytes to be send. Do not
     * send packets larger than 1514 bytes.
     *
     * The previous version was wrong by specifying a maximum
     * of 1518, because it didn't take the CRC into account,
     * which is generated by the hardware and automatically
     * appended. Thanks to Bengt Florin, who discovered this.
     */
    if (size > 1514)
        return -1;

    /*
     * The controller will not append pad bytes,
     * so we have to do this.
     */
    if (size < 60) {
        padding = (u_char) (60 - p->tot_len);
        size = 60;
    }

    /*
     * Bengt Florin introduces polling mode for the transmitter. Be
     * aware, that this may introduce other problems. If a high
     * priority thread is waiting for the transmitter, it may hold
     * the CPU for more than 1.2 milliseconds in worst cases.
     */
     
    /*
     while (NICINB(NIC_CR) & NIC_CR_TXP)
        NutThreadYield();
    */

    /* we don't want to be interrupted by NIC owerflow */
    //irq_context=alt_irq_disable_all();

    /*
     * Set remote dma byte count
     * and start address.
     */
    IOWR_CTRL_RTL8019_RBCR0(dev->base_addr, size);
    IOWR_CTRL_RTL8019_RBCR1(dev->base_addr, size >> 8);
    IOWR_CTRL_RTL8019_RSAR0(dev->base_addr,0);
    IOWR_CTRL_RTL8019_RSAR1(dev->base_addr,NIC_FIRST_TX_PAGE);

    /*
     * Peform the write.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr, CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD1_MSK);

    /*
     * Transfer the Ethernet frame.
     */
    for(q = p; q != NULL; q = q->next) 
    {
        NicWrite(dev->base_addr,q->payload, q->len);
    }
    
    /*
     * Add pad bytes.
     */
    for (i = 0; i < padding; i++)
        IOWR_CTRL_RTL8019_DATA(dev->base_addr,0);

    /*
     * Complete remote dma.
     */
    NicCompleteDma(dev->base_addr);

    /*
     * Number of bytes to be transmitted.
     */
    IOWR_CTRL_RTL8019_TBCR0(dev->base_addr,(size & 0xff));
    IOWR_CTRL_RTL8019_TBCR1(dev->base_addr,((size >> 8) & 0xff));

    /*
     * First page of packet to be transmitted.
     */
    IOWR_CTRL_RTL8019_TPSR(dev->base_addr,NIC_FIRST_TX_PAGE);

    /*
     * Start transmission.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_TXP_MSK | CTRL_RTL8019_CR_RD2_MSK);

    LWIP_DEBUGF(NETIF_DEBUG | DBG_TRACE, ("low_level_ouptput () return OK\n"));
    return 0;
}


/*
 * \brief Handle NIC overflows.
 *
 * When a receiver buffer overflow occurs, the NIC will defer any subsequent
 * action until properly restarted.
 *
 * This routine is called within interrupt context, which introduces a big
 * problem. It waits for the last transmission to finish, which may take
 * several milliseconds. Since Nut/OS 3.5, we do not support nested interrupts
 * on AVR systems anymore. So this routine may now increase interrupt
 * latency in an unacceptable way. The solution might be to handle overflows
 * in the receiver thread.
 *
 * In any case, this routines needs a major redesign. But it has been
 * tested in its current form to gracefully withstand ping floods. Thanks
 * to Bengt Florin for contributing his code, which provides much more
 * stability than its predecessor.
 */
u_char NicOverflow(ctrl_rtl8019_if *dev)
{
    u_char cr;
    u_char resend = 0;
    u_char curr;

    /*
     * Wait for any transmission in progress. Save the command register,
     * so we can later determine, if NIC transmitter has been interrupted.
     * or reception in progress.
     */
    while (IORD_CTRL_RTL8019_CR(dev->base_addr) & CTRL_RTL8019_CR_TXP_MSK);
    cr = IORD_CTRL_RTL8019_CR(dev->base_addr);

    /*
     * Get the current page pointer. It points to the page where the NIC
     * will start saving the next incoming packet.
     */
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STP_MSK | CTRL_RTL8019_CR_RD2_MSK | CTRL_RTL8019_CR_PS0_MSK);
    curr = IORD_CTRL_RTL8019_CURR(dev->base_addr);
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STP_MSK | CTRL_RTL8019_CR_RD2_MSK);

    /* Clear remote byte count register. */
    IOWR_CTRL_RTL8019_RBCR0(dev->base_addr,0);
    IOWR_CTRL_RTL8019_RBCR1(dev->base_addr,0);

    /* Check for any incomplete transmission. */
    if ((cr & CTRL_RTL8019_CR_TXP_MSK) && ((IORD_CTRL_RTL8019_ISR(dev->base_addr) & (CTRL_RTL8019_ISR_PTX_MSK | CTRL_RTL8019_ISR_TXE_MSK)) == 0))
    {
        resend = 1;
    }

    /* Enter loopback mode and restart the NIC. */
    IOWR_CTRL_RTL8019_TCR(dev->base_addr,CTRL_RTL8019_TCR_LB0_MSK);
    IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_RD2_MSK);

    /*
     * Discard all packets from the receiver buffer. Set boundary
     * register to the last page we read.
     */
    if (--curr < NIC_FIRST_RX_PAGE) {
        curr = NIC_STOP_PAGE - 1;
    }
    IOWR_CTRL_RTL8019_BNRY(dev->base_addr,curr);

    /* Switch from loopback to normal mode mode. */
    IOWR_CTRL_RTL8019_TCR(dev->base_addr,0);

    /* Re-invoke any interrupted transmission. */
    if (resend) {
        IOWR_CTRL_RTL8019_CR(dev->base_addr,CTRL_RTL8019_CR_STA_MSK | CTRL_RTL8019_CR_TXP_MSK | CTRL_RTL8019_CR_RD2_MSK);
    }

    /* Finally clear the overflow flag */
    IOWR_CTRL_RTL8019_ISR(dev->base_addr,CTRL_RTL8019_ISR_OVW_MSK);
    return resend;
}

/*
 * \brief NIC interrupt entry.
 */
void ctrl_rtl8019_irq(void* context, alt_u32 interrupt)
{
    u_char isr;
    
    int rx_frame_errors;
    int rx_crc_errors;
    int rx_missed_errors;
    ctrl_rtl8019_if *dev =  (ctrl_rtl8019_if *)context;
    
    isr = IORD_CTRL_RTL8019_ISR(dev->base_addr);
    IOWR_CTRL_RTL8019_ISR(dev->base_addr,isr);

    /*
     * Recover from receive buffer overflow. This may take some
     * time, so we enable global interrupts but keep NIC
     * interrupts disabled.
     */
    if (isr & CTRL_RTL8019_ISR_OVW_MSK) {
        NicOverflow(dev);
    } else {
        /*
         * If this is a transmit interrupt, then a packet has been sent.
         * So we can clear the transmitter busy flag and wake up the
         * transmitter thread.
         */
        if (isr & CTRL_RTL8019_ISR_TXE_MSK);

        /*
         * If this is a receive interrupt, then wake up the receiver
         * thread.
         */
        if (isr & CTRL_RTL8019_ISR_PRX_MSK){
        	sys_mbox_post(rx_mbox, &dev->lwip_dev_list.dev);
        }

        if (isr & CTRL_RTL8019_ISR_RXE_MSK) {
            rx_frame_errors += IORD_CTRL_RTL8019_CNTR0(dev->base_addr);
            rx_crc_errors += IORD_CTRL_RTL8019_CNTR1(dev->base_addr);
            rx_missed_errors += IORD_CTRL_RTL8019_CNTR2(dev->base_addr);
        }
    }
}

/*-----------------------------------------------------------------------------------* 
 *
 * ctrl_rtl8019_output():
 *
 * This function is called by the TCP/IP stack when an IP packet
 * should be sent. It calls the function called low_level_output() to
 * do the actuall transmission of the packet.
 *
 * 
 *-----------------------------------------------------------------------------------*/
static err_t
ctrl_rtl8019_output(struct netif *netif, struct pbuf *p,
      struct ip_addr *ipaddr)
{
  err_t err;
  ctrl_rtl8019_if* dev = (ctrl_rtl8019_if*)netif->state;

  sys_sem_wait(dev->arp_semaphore);

  /* 
  * resolve hardware address, then send (or queue) packet 
  * The code which updates the ARP tables does not appear to be thread safe
  * so I've added a MUTEX around all calls to the arp code
  * 
  */
  
  err = etharp_output(netif, ipaddr, p);
  sys_sem_signal(dev->arp_semaphore);

  return err;
}

#endif

