#ifndef _PCF8563_H_
#define _PCF8563_H_

#define STATUS1           0x00
#define STATUS2           0x00
#define SECONDS           0x30
#define MINUTES           0x55
#define HOURS             0x13
#define DAYS              0x14
#define WEEKDAYS          0x1
#define Months_Century    0x11
#define Years             0x07
#define Minute_alarm      0x00
#define Hour_alarm        0x00
#define Day_alarm         0x00
#define Weekday_alarm     0x00
#define CLKOUT_frequency  0x80
#define Timer_control     0x83
#define Timer_countdown_value  0x00

extern void I2CWaitTIP();

void PCF8563_WriteReg(alt_u32 base, alt_u8 address,alt_u8 reg, alt_u8 *buf, alt_u16 num)
{
  alt_u16 i,tmp;
  // Wait for the completion of transfer
   usleep(5000);
  I2CWaitTIP(base);

  // write address of I2C slave device
  // and generate START & WR command
  IOWR_OC_I2C_TXR(base, address);
  IOWR_OC_I2C_CR(base, I2C_CR_STA | I2C_CR_WR);
  I2CWaitTIP(base);

  // write register address
  IOWR_OC_I2C_TXR(base, reg);
  IOWR_OC_I2C_CR(base, I2C_CR_WR);
  I2CWaitTIP(base);

  // write data
  if(num > 0) 
  {
    tmp = num - 1;
    for(i=0; i<tmp; i++)
    {
       IOWR_OC_I2C_TXR(base,*buf++);
       IOWR_OC_I2C_CR(base, I2C_CR_WR);
       I2CWaitTIP(base);
       usleep(5000);
    }
  }
  // write data with STOP signal
  IOWR_OC_I2C_TXR(base, *buf);
  IOWR_OC_I2C_CR(base, I2C_CR_WR | I2C_CR_STO);
  I2CWaitTIP(base);
}

unsigned char I2CReaddata(alt_u32 base,alt_u8 address,alt_u8 reg)//, alt_u16 num)
{
  alt_u8 data;
  // Wait for the completion of transfer
   usleep(5000);
  I2CWaitTIP(base);
 
  // write address of I2C slave device
  // and generate START & WR command
  IOWR_OC_I2C_TXR(base, address);
  IOWR_OC_I2C_CR(base, I2C_CR_STA | I2C_CR_WR);
  I2CWaitTIP(base);

  // write register address
  IOWR_OC_I2C_TXR(base, reg);
  IOWR_OC_I2C_CR(base, I2C_CR_WR);
  I2CWaitTIP(base);

  // write address of I2C slave device
  // and RD command
  IOWR_OC_I2C_TXR(base, address|1);
  IOWR_OC_I2C_CR(base, I2C_CR_STA | I2C_CR_WR);
  I2CWaitTIP(base);
  /*
  // Read data
  if(num > 0) 
  {
    tmp = num - 1;
    for(i=0; i<tmp; i++)
    {
       IOWR_OC_I2C_CR(base,I2C_CR_RD | I2C_CR_ACK); 
       I2CWaitTIP(base);
       *buf++ = IORD_OC_I2C_RXR(base);
       usleep(5000);
    }
  }
*/
  // Read data with STOP signal
  IOWR_OC_I2C_CR(base,I2C_CR_RD | I2C_CR_ACK | I2C_CR_STO); 
  I2CWaitTIP(base);
  data= IORD_OC_I2C_RXR(base);
  return data;
}


void PCF8563_SET(void)
 {
  alt_u8 content[16]               ;
    *content=STATUS1               ;
    *(content+1) = STATUS2            ;
    *(content+2) = SECONDS            ;
    *(content+3) = MINUTES            ;
    *(content+4) = HOURS              ;
    *(content+5) = DAYS               ;
    *(content+6) = WEEKDAYS           ;
    *(content+7) = Months_Century     ;
    *(content+8) = Years              ;
    *(content+9) = Minute_alarm       ;
    *(content+10) = Hour_alarm        ;
    *(content+11) = Day_alarm         ;
    *(content+12) = Weekday_alarm     ;
    *(content+13) = CLKOUT_frequency  ;
    *(content+14) = Timer_control     ;
    *(content+15) = Timer_countdown_value ;
  
PCF8563_WriteReg(OC_I2C_MASTER_0_BASE,0xa2,0x0,content,16)  ;
 }
#endif //_PCF8563_H_
