/******************************************************************************
* Copyright ? 2004 Altera Corporation, San Jose, California, USA.             *
* All rights reserved. All use of this software and documentation is          *
* subject to the License Agreement located at the end of this file below.     *
*******************************************************************************
* Author - S. O'Reilly                                                        *
* Date - July 4, 2004                                                         *
* Module - network_utilities.c                                                *
*                                                                             *
* Based on Telnet example by JRK and lwIP & HTTP server ports by PRR          *
******************************************************************************/

/* 
 * Simple Socket Server (SSS) example. 
 * 
 * Please refer to the Nios II lwIP Tutorial documentation for details on this 
 * software example, as well as details on how to configure the lwIP TCP/IP 
 * networking stack and MicroC/OS-II Real-Time Operating System.  The Nios II
 * lwIP Tutorial, along with the rest of the Nios II documentation is published 
 * on the Altera web-site.  See: 
 * Start Menu -> Programs -> Nios II Development Kit -> Nios II Documentation.
 */
 
/*
 * Include files:
 * 
 *      <stdio.h>: Standard IO
 *     includes.h: This is the default MicroC/OS-II include file.
 * alt_lwip_dev.h: Defines lwIP stack initialization calls.
 *    alt_types.h: Nios II CPU data type definitions.
 *           io.h: Nios II CPU I/O access macro definitions.
 *       simple_socket_server.h: Our simple socket server app's declarations.
 * alt_error_handler.h: Altera Error Handler suite of development error 
 * handling functions.
 */
#include <stdio.h> 
#include <string.h> 
#include <ctype.h>
#include <alt_lwip_dev.h>
#include <alt_types.h>
#include <sys/alt_flash.h>
#include "includes.h"
#include "io.h"
#include "simple_socket_server.h"
#include "alt_error_handler.h"

/*
 * The "adapter" storage here is declared so that when lwIP calls the 
 * get_ip_addr() routine, the adapter struct will be populated with the 
 * network device (Ethernet MAC) that lwIP is currently trying to setup an IP
 * address for. This is later used in the dhcp_timeout_task() routine to 
 * determine if DHCP was successful for the particular adapter.
 */
struct netif* adapter;

/*
 * Handle to our Nios Development board LCD panel.
 */
FILE *LCDDevice;

/*
 * generate_and_store_mac_addr()
 * 
 * This routine is called when, upon program initialization, we discover
 * that there is no valid network settings (including MAC address) programmed
 * into flash memory at the last flash sector.  If it is not safe to use the 
 * contents of this last sector of flash, the user is prompted to
 * enter the serial number at the console.  A MAC address is then
 * generated using 0xFF followed by the last 2 bytes of the serial number 
 * appended to Altera's Vendor ID, an assigned MAC address range with the first
 * 3 bytes of 00:07:ED.  For example, if the Nios Development Board serial 
 * number is 040800017, the corresponding ethernet number generated will be
 * 00:07:ED:FF:8F:11.
 * 
 * It should be noted that this number, while unique, will likely differ from
 * the also unique (but now lost forever) MAC address programmed into the 
 * development board on the production line.
 * 
 * As we are erasing the entire flash sector, we'll re-program it with not
 * only the MAC address, but static IP, subnet, gateway, and "Use DHCP" 
 * sections. These fail-safe static settings are compatible with previous
 * Nios Ethernet designs, and allow the "factory-safe" design to behave 
 * as expected if the last flash sector is erased.
 */
err_t generate_and_store_mac_addr()
{
  err_t ret_code = ERR_IF;
  alt_u32 ser_num = 0;
  char serial_number[9]="123456789";
  char flash_content[32];
  alt_flash_fd* flash_handle;
  int i = 0;
  
  printf("Can't read the MAC address from your board (this probably means\n");
  printf("that your flash was erased). We will assign you a MAC address and\n");
  printf("static network settings\n\n");
  
  while(!ser_num)
  {
    printf("Please enter your 9-digit serial number. This is printed on a \n");
    printf("label under your Nios dev. board. The first 3 digits of the \n");
    printf("label are ASJ and the serial number follows this.\n -->");
    /*
    for(i=0; i<9; i++)
    {
      serial_number[i] = getchar();
      putchar(serial_number[i]);
      
      // Handle backspaces.  How civilized. 
      if ((serial_number[i] == 0x08) && (i >= 0)) 
      {
        i--;
      }
    }
    printf("\n");*/
  //  serial_number[9] = {1,2,3,4,5,6,7,8,9,};
            
    for(i=0; i<9; i++)
    {
      if (isdigit(serial_number[i]))
      {
        ser_num *= 10;
        ser_num += serial_number[i] - '0';
      }
      else
      {
        ser_num = 0;
        printf("Serial number only contains decimal digits and is non-zero\n");
        break;
      }
    }
    
    if (ser_num)
    {
      /* This says the image is safe */
      flash_content[0] = 0xfe;
      flash_content[1] = 0x5a;
      flash_content[2] = 0x0;
      flash_content[3] = 0x0;
      
      /* This is the Altera Vendor ID */
      flash_content[4] = 0x0;
      flash_content[5] = 0x7;
      flash_content[6] = 0xed;
      
      /* Reserverd Board identifier for erase boards */
      flash_content[7] = 0xFF;
      flash_content[8] = (ser_num & 0xff00) >> 8;
      flash_content[9] = ser_num & 0xff;

      /* Then comes a 16-bit "flags" field (not used with lwIP) */
      flash_content[10] = 0xFF;
      flash_content[11] = 0xFF;
      
      /* Then comes the static IP address */
      flash_content[12] = IPADDR0;
      flash_content[13] = IPADDR1;
      flash_content[14] = IPADDR2;
      flash_content[15] = IPADDR3;
      
      /* Then comes the static nameserver address (not used with lwIP)  */
      flash_content[16] = 0xFF;
      flash_content[17] = 0xFF;
      flash_content[18] = 0xFF;
      flash_content[19] = 0xFF;
      
      /* Then comes the static subnet mask */
      flash_content[20] = MSKADDR0;
      flash_content[21] = MSKADDR1;
      flash_content[22] = MSKADDR2;
      flash_content[23] = MSKADDR3;
      
      /* Then comes the static gateway address */
      flash_content[24] = GWADDR0;
      flash_content[25] = GWADDR1;
      flash_content[26] = GWADDR2;
      flash_content[27] = GWADDR3;
      
      /* And finally whether to use DHCP - set all bits to be safe */
      flash_content[28] = 0xFF;
      flash_content[29] = 0xFF;
      flash_content[30] = 0xFF;
      flash_content[31] = 0xFF;
      
      /* Write the MAC address to flash */
      flash_handle = alt_flash_open_dev(FLASH_NAME);
      if (flash_handle)
      {
        alt_write_flash(flash_handle, LAST_SECTOR_OFFSET, flash_content, 32);
        alt_flash_close_dev(flash_handle);
        ret_code = ERR_OK;
      }
    }
  }
  return ret_code;    
}

/*
* get_mac_addr
*
* Read the MAC address in a board specific way
*
*/
err_t get_mac_addr(alt_lwip_dev* lwip_dev)
{
  err_t ret_code = ERR_OK;
  alt_u32 signature;
  struct netif* netif = lwip_dev->netif;


#if  defined(FLASH_NAME)
  signature = IORD_32DIRECT(LAST_FLASH_SECTOR, 0);
  
/* This LAST_FLASH_SECTOR region of flash is examined to see if 
 * valid network settings are present, indicated by a signature of 0x00005afe at 
 * the first address of the last flash sector.  This hex value is chosen as the 
 * signature since it looks like the english word "SAFE", meaning that it is 
 * safe to use these network address values.  
*/
  if (signature != 0x00005afe)
  {
    ret_code = generate_and_store_mac_addr();
  }
  
  if (ret_code == ERR_OK)
  {
    netif->hwaddr[0] = IORD_8DIRECT(LAST_FLASH_SECTOR, 4);
    netif->hwaddr[1] = IORD_8DIRECT(LAST_FLASH_SECTOR, 5);
    netif->hwaddr[2] = IORD_8DIRECT(LAST_FLASH_SECTOR, 6);
    netif->hwaddr[3] = IORD_8DIRECT(LAST_FLASH_SECTOR, 7);
    netif->hwaddr[4] = IORD_8DIRECT(LAST_FLASH_SECTOR, 8);
    netif->hwaddr[5] = IORD_8DIRECT(LAST_FLASH_SECTOR, 9);
    
    printf("Your Ethernet MAC address is %02x:%02x:%02x:%02x:%02x:%02x\n", 
            netif->hwaddr[0],
            netif->hwaddr[1],
            netif->hwaddr[2],
            netif->hwaddr[3],
            netif->hwaddr[4],
            netif->hwaddr[5]);

  }
#else
  ret_code = ERR_IF;

  perror( "Not an Altera Nios Development Board.\n");
  perror( "You need to modify the function get_mac_addr \n");
  perror( "to set a MAC address for your board");
#endif

  return ret_code;
}

/*
 * get_ip_addr()
 * 
 * This routine is called by lwIP to obtain an IP address for the specified 
 * network adapter. Like the MAC address, obtaining an IP address is very
 * system-dependant and therefore this function is exported for the developer
 * to control.
 * 
 * In our system, we are either attempting DHCP auto-negotiation of IP address,
 * or we are setting our own static IP, Gateway, and Subnet Mask addresses our
 * self. This routine is where that happens.
 */
int get_ip_addr(alt_lwip_dev* lwip_dev, struct ip_addr* ipaddr, 
                struct ip_addr* netmask, struct ip_addr* gw,
                int* use_dhcp)
{
  int ret_code = 0;  

#ifdef LCD_TRI_1602_0_NAME
     LCDDevice = fopen(LCD_TRI_1602_0_NAME, "w");
     if (LCDDevice <= 0) 
     {
         alt_lwIPErrorHandler(EXPANDED_DIAGNOSIS_CODE,
                  "[network_utilities:get_ip_addr] fopen LCD_DISPLAY_NAME failed");
     }
#endif /* LCD_DISPLAY_NAME */

  if (!strcmp(lwip_dev->name, "/dev/" LWIP_DEFAULT_IF))
  {
#if LWIP_DHCP == 1
    *use_dhcp = 1;

    /*
     * If we are telling lwIP to attempt DHCP, all that is needed is to save 
     * the lwIP device in question to a the global netif "adapter"; the 
     * DHCP timeout task will then query this device to see whether an IP
     * address has been set.
     * 
     * If we are not attempting DHCP, static network settings are assigned here
     * and we go on our merry way.
     */
    adapter = lwip_dev->netif;

#ifdef LCD_DISPLAY_NAME  
    /* Clear the LCD screen */
    fprintf(LCDDevice, "\x1b");
    fprintf(LCDDevice, "[2J");
    fprintf(LCDDevice, "Using DHCP to\nfind IP Addr");
#endif /* LCD_DISPLAY_NAME */
    printf("Using DHCP to find an IP Address\n");

#else
    IP4_ADDR(ipaddr, IPADDR0,IPADDR1,IPADDR2,IPADDR3);
    IP4_ADDR(gw, GWADDR0,GWADDR1,GWADDR2,GWADDR3);
    IP4_ADDR(netmask, MSKADDR0,MSKADDR1,MSKADDR2,MSKADDR3);
    *use_dhcp = 0;

    printf("Static IP Address is %d.%d.%d.%d\n", 
        ip4_addr1(ipaddr),
        ip4_addr2(ipaddr),
        ip4_addr3(ipaddr),
        ip4_addr4(ipaddr));

#ifdef LCD_TRI_1602_0_NAME 
      /* Clear the LCD screen */
   //   fp=fopen(LCD_TRI_1602_NAME ,"w");
      fprintf(LCDDevice,"Static IP Addr:1");
      fprintf(LCDDevice,"%d.%d.%d.%d2",
        ip4_addr1(ipaddr),
        ip4_addr2(ipaddr),
        ip4_addr3(ipaddr),
        ip4_addr4(ipaddr));
      /*
        ip4_addr1(&adapter->ip_addr),
        ip4_addr2(&adapter->ip_addr),
        ip4_addr3(&adapter->ip_addr),
        ip4_addr4(&adapter->ip_addr));*/
#endif /* LCD_DISPLAY_NAME */


#endif /* lwIP_DHCP */



    ret_code = 1;
  }

  return ret_code;
}

/*
 * NETUTILSDHCPTimeoutTask()
 * 
 * This lwIP-called routine is responsible for checking to see whether DHCP 
 * was successful. Here we check to see whether a timeout (120 seconds in this
 * case) has elapsed. If so, display an error, assign a static IP address, and 
 * bail out. If not, continue checking for DHCP success and bail out once
 * a DHCP address assignment is complete.
 * 
 * This routine also displays the assigned IP address to the user (via STDOUT 
 * and, if equipped, LCD screen.
 * 
 * Note: LCD_DISPLAY_NAME is defined in telnet.h to be the LCD interface for
 *       the "standard" and "full-featured" example designs.
 */
#if LWIP_DHCP == 1
void NETUTILSDHCPTimeoutTask()
{
  int have_address = 0, timeout_dhcp = 0;
  struct ip_addr ipaddr, netmask, gw;
  INT8U error_code;
  
  error_code = OS_NO_ERR;
  
  while(1)
  {
    OSTimeDlyHMSM(0,0,0,100);

    /* 
     * For demo purposes we need to use DHCP to allocate an IP Address
     * and lwIP had no mechanism to tell us one is allocated. Wait two 
     * minutes if we don't have one we use a static IP address
     */
    if((timeout_dhcp < DHCP_TIMEOUT) && (have_address == 0))
    {
      timeout_dhcp++;
      if (!ip_addr_isany(&adapter->ip_addr))
      {
        have_address = 1;
        printf("Assigned IP Address is %d.%d.%d.%d\n\n",
                ip4_addr1(&adapter->ip_addr),
                ip4_addr2(&adapter->ip_addr),
                ip4_addr3(&adapter->ip_addr),
                ip4_addr4(&adapter->ip_addr));

#ifdef LCD_DISPLAY_NAME        
        /* Clear screen */
      //  fprintf(LCDDevice, "\x1b");
      //  fprintf(LCDDevice, "[2J");
        fprintf(LCDDevice, "DHCP IP Addr is \n %d.%d.%d.%d",
                ip4_addr1(&adapter->ip_addr),
                ip4_addr2(&adapter->ip_addr),
                ip4_addr3(&adapter->ip_addr),
                ip4_addr4(&adapter->ip_addr));
#endif /* LCD_DISPLAY_NAME */
      }
    }
    /*
     * You're in a bit of trouble if this happens, you could assign a static
     * IP address but how do we chose it?
     */
    if (timeout_dhcp ==  DHCP_TIMEOUT)
    {
      dhcp_stop(adapter);
      IP4_ADDR(&ipaddr, IPADDR0,IPADDR1,IPADDR2,IPADDR3);
      IP4_ADDR(&gw, GWADDR0,GWADDR1,GWADDR2,GWADDR3);
      IP4_ADDR(&netmask, MSKADDR0,MSKADDR1,MSKADDR2,MSKADDR3);
      
      netif_set_addr( adapter,&ipaddr, &netmask, &gw);
      netif_set_up(adapter);
      
      printf("DHCP Failed to assign an IP Address\n\n"); 
      printf("\nUsing static address: %d.%d.%d.%d\n\n\n",
        ip4_addr1(&adapter->ip_addr),
        ip4_addr2(&adapter->ip_addr),
        ip4_addr3(&adapter->ip_addr),
        ip4_addr4(&adapter->ip_addr));
/*
#ifdef LCD_DISPLAY_NAME  
      // Clear the LCD screen 
      fprintf(LCDDevice, "\x1b");
      fprintf(LCDDevice, "[2J");
      fprintf(LCDDevice, "DHCP Failed using\n %d.%d.%d.%d",
        ip4_addr1(&adapter->ip_addr),
        ip4_addr2(&adapter->ip_addr),
        ip4_addr3(&adapter->ip_addr),
        ip4_addr4(&adapter->ip_addr));
#endif // LCD_DISPLAY_NAME 
*/
#ifdef LCD_TRI_1602_0_NAME 
      /* Clear the LCD screen */
      LCDDevice=fopen(LCD_TRI_1602_0_NAME ,"w");
      fprintf(LCDDevice,"The IP Address:1");
      fprintf(LCDDevice,"%d.%d.%d.%d2",
        ip4_addr1(&adapter->ip_addr),
        ip4_addr2(&adapter->ip_addr),
        ip4_addr3(&adapter->ip_addr),
        ip4_addr4(&adapter->ip_addr));
#endif /* LCD_DISPLAY_NAME */

      have_address = 1;
    }
    
    if (have_address == 1)
    {
      /* 
       * We have an IP address this task has served it's purpose - kill it.
       */
#ifdef LCD_TRI_1602_0_NAME        
       fclose(LCDDevice);
#endif /* LCD_DISPLAY_NAME */
       error_code = OSSemPost(SSSAttainedIPAddressSem);
       alt_uCOSIIErrorHandler(error_code, 0);

       OSTaskDel(OS_PRIO_SELF);
    }
  }
}
#endif /* LWIP_DHCP */


/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2004 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
