/*
 * Filename: ipport.h
 *
 * Copyright 2004 InterNiche Technologies Inc. All rights reserved.
 *
 * All the platform specific defines for each port go here.
 * All these should be re-examined with each new port of the code.
 *
 * This file for:
 *   ALTERA Cyclone Dev board with the ALTERA Nios2 Core.
 *   SMSC91C11 10/100 Ethernet Controller
 *   GNU C Compiler provided by ALTERA Quartus Toolset.
 *   Quartus HAL BSP
 *   uCOS-II RTOS Rel 2.76 as distributed by Altera/NIOS2
 *
 * 06/21/2004
 *
 */

#ifndef _IPPORT_H_
#define _IPPORT_H_ 1

#ifdef ALT_INICHE
#include "system.h"
#endif

/* TTYIO.C has redefintions of printf, sprintf. So to resolve declarations,
 * we don't include stdio.h for compiling this file
 */
#ifndef _IN_TTYIO_
#include "stdio.h"   /* C compiler files */
#endif

#include "stdlib.h"
#include "stdarg.h"
#include "string.h"

#define UINT_ALREADY
#include "nptypes.h"   /* NetPort data types */

#ifndef TRUE
#define TRUE   -1
#endif

#ifndef FALSE
#define FALSE   0
#endif

#ifndef   NULL
#define   NULL ((void*)0)
#endif   /* NULL */


#ifndef cticks
extern unsigned long cticks; /* clock ticks since startup */
#else
extern unsigned long clock( void );
#endif
void   clock_init(void);     /* start clock tick counting; called at init time */
void   clock_c(void);        /* undo effects of clock_init (ie: restore ISR vector */

#define TPS   20      /* cticks per second */
#define CMPTIME(T,C)  (((T)<=(C)) && (((C)-(T))<(60*TPS)))
#define HISTORY(T)    CMPTIME((T),cticks)

/* define preferred data buffer boundary, usually 2 or 4 */
#define ALIGN_TYPE   4  /* 32 bit alignment */

/* 
 * convert little/big endian - these should be efficient, 
 * inline code or MACROs 
 *
 * Note: __BIG_ENDIAN and/or __BigEndian macros appear to be 
 * auto-defined by Green Hills C in big-endian mode.
 */
#if defined(__BIG_ENDIAN) || defined(__BigEndian)
#define htonl(l) (l)
#define ntohl(l) (l)
#define htons(s) (s)
#define ntohs(s) (s)
#else
#define lswap(x) ((((x) & 0xff000000) >> 24) | \
		  (((x) & 0x00ff0000) >>  8) | \
		  (((x) & 0x0000ff00) <<  8) | \
		  (((x) & 0x000000ff) << 24))
#define htonl(l) (lswap(l))
#define ntohl(l) (lswap(l))
#define htons(s) ((((s) >> 8) & 0xff) | \
		  (((s) << 8) & 0xff00))
#define ntohs(s) htons(s)
#endif

/* definitions for the BSD TCP code */
#define   LITTLE_ENDIAN 1234
#define   BIG_ENDIAN    4321
#ifdef __BIG_ENDIAN
#define   BYTE_ORDER    BIG_ENDIAN
#else
#define   BYTE_ORDER    LITTLE_ENDIAN
#endif

#include "string.h"   /* from C lib, for memmove(), et al */

/* Options for setting the MicroC/OS-II priorities and stack sizes
 * for the various tasks used by the stack 
 */

/* task stack sizes */
#define  DEMO_STACK_SIZE      (4096+8192)

#define  NETHEAP_SIZE         0x30000

/* task stack sizes */
#define  NET_STACK_SIZE       (4096+8192)
#define  APP_STACK_SIZE       (6144+8192) /* 4096 */     /* default for applications */
#define  CLOCK_STACK_SIZE     (4096+8192)

#define  IO_STACK_SIZE        (2048+8192)
#define  WEB_STACK_SIZE       (APP_STACK_SIZE+8192)
#define  FTP_STACK_SIZE       APP_STACK_SIZE
#define  PING_STACK_SIZE      (4096+8192)
#define  TN_STACK_SIZE        APP_STACK_SIZE
#define  TCP_ECHO_STACK_SIZE  APP_STACK_SIZE

// TK_NETMAIN and TK_NETTICK are always used by NicheStack
// These priorities must be defined
#define TK_NETMAIN_TPRIO 2
#define TK_NETMAIN_SSIZE NET_STACK_SIZE
#define TK_NETTICK_TPRIO 3
#define TK_NETTICK_SSIZE CLOCK_STACK_SIZE

#define TK_PINGCHECK_TPRIO 5
#define TK_PINGCHECK_SSIZE PING_STACK_SIZE
#define TK_ECHOTEST_TPRIO 5
#define TK_ECHOTEST_SSIZE TCP_ECHO_STACK_SIZE
#define TK_CIPHERTEST_TPRIO 5
#define TK_CIPHERTEST_SSIZE APP_STACK_SIZE
#define TK_BROWCHECK_TPRIO 5
#define TK_BROWCHECK_SSIZE (APP_STACK_SIZE)*4
#define TK_CUEXECUTION_TPRIO 5
#define	TK_CUEXECUTION_SSIZE APP_STACK_SIZE
#define TK_WEBPORT_TPRIO 5
#define	TK_WEBPORT_SSIZE WEB_STACK_SIZE 
#define TK_FTPSRVR_TPRIO 5
#define TK_FTPSRVR_SSIZE FTP_STACK_SIZE
#define TK_FTPCLNT_TPRIO 5
#define TK_FTPCLNT_SSIZE FTP_STACK_SIZE
#define TK_EMAILER_TPRIO 5
#define TK_EMAILER_SSIZE APP_STACK_SIZE 
#define TK_TELNETSRV_TPRIO 5
#define TK_TELNETSRV_SSIZE APP_STACK_SIZE
#define TK_SNMP_TPRIO 5
#define TK_SNMP_SSIZE APP_STACK_SIZE 
#define TK_DNSSRV_TPRIO 5
#define TK_DNSSRV_SSIZE APP_STACK_SIZE 
#define TK_IKE_TK_PRIO 5
#define TK_IKE_SSIZE APP_STACK_SIZE 
#define TK_KEYBOARD_TPRIO 5
#define TK_KEYBOARD_SSIZE IO_STACK_SIZE
#define TK_SYSLOG_TPRIO 5
#define TK_SYSLOG_SSIZE APP_STACK_SIZE


/* Options to trade off features for size. Do not enable options 
 * for modules you don't have or your link will get unresolved 
 * externals.
 */

#define INCLUDE_ARP     1  /* use Ethernet ARP */
#define FULL_ICMP       1  /* use all ICMP || ping only */
#define NB_CONNECT      1  /* support Non-Blocking connects (TCP, PPP, et al) */
#define MUTE_WARNS      1  /* gen extra code to suppress compiler warnings */
#define NPDEBUG         1  /* turn on debugging dprintf()s */
#define NATIVE_PRINTF   1  /* use target build environment's printf function */
#define PRINTF_STDARG   1  /* build ...printf() using stdarg.h */
#define MULTI_HOMED     1  /* more than one net interface */
#define BSD_SOCKETS     1  /* use BSD sockets API */
#define IP_RAW          1  /* support raw sockets */


/*
 * A place to keep disabled options.
 */
#ifdef NOT_USED
#error Make sure "NOT_USED" is really undefined
#define INICHE_TASKS    1  /* map tasking to InterNiche NicheTask */
#define SMSC_POLLED     1  /* Build SMSC Drivers in Polled Mode */
#define C_CHECKSUM      1  /* Use C Check Routine in .\misclib directory */
#define MEM_WRAPPERS    1  /* wrap memory allocate/free functions */
#define WEBPORT         1  /* will be linked with WebPort Web (HTTP) Server */
#define HTML_COMPRESSION     1  /* TRUE to include HTML tags compression logic */
#define FOREIGN_LANG_SUPPORT 1  /* TRUE to compress around high bit chars */
#define USE_PROFILER    1  /* profile selected code modules */
#define SERVER_PUSH     1  /* Web server "push" code */
#define INICHE_TIMERS   1  /* Provide Interval timers */
#define ROUTE_TEST      1  /* Pseudo device to test IP/NAT routing */
#define TCP_SACK        1  /* Selective ACK per rfc2018 */
#define LOSSY_IO        1  /* sabotage packet IO for testing */
#define IP_V6           1  /* Include IP version 6 in build */
#define LINKED_PKTS     1  /* scatter/gather packets */
#define BTREE_ROUTES    1  /* keep IP routes in btree for fast lookup */
#define IP_MULTICAST    1  /* support end node IP multicast */
#define IPV6_TUNNEL     1  /* Generic IPv6 tunnels */
#define INCLUDE_6TO4    1  /* RFC-3056 6to4 tunnel */
#define IEEE_802_3      1  /* Support IEEE 802.3 (RFC-1042) */
#define IP_PMTU         1  /* do path MTU discovery */
#define DNS_SERVER      1  /* include DNS server code */
#define SMTP_ALERTS     1  /* send Emailer trouble alerts */
#define DHCP_SERVER     1  /* bring in DHCP server */
#define HT_USERAUTH     1  /* TRUE to include user authorization code */
#define HT_MD5AUTH      1  /* TRUE to include MD5 code */
#define DYNAMIC_IFACES  1  /* runtime iface creation/deletion */
#define NATRT           1  /* enable NAT routing feature */
#define RIP_SUPPORT     1  /* enable RIP routing feature */
#define INCLUDE_SNMPV1  1  /* SNMPv1 library, agent & hook */
#define INCLUDE_SNMPV2C 1  /* SNMPv2c (community based SNMPv2) agent */
#define INCLUDE_SNMPV3  1  /* SNMPv3 library, agent & hook */
#define POP3_CLIENT     1  /* POP3 client API code */
#define P3C_APOP        1  /* POP3 client APOP supporting code */
#define UDPSTEST        1  /* UDP Echo Sockets test app */
#define TCP_ECHOTEST    1  /* include a TCP Echo test */
#define BOOTPTAB        1  /* DHCP supports a UNIX-ish bootptab file */
#define USE_SLIP        1  /* support SLIP driver iface(s) */
#define NO_UDP_CKSUM    1  /* don't do UDP checksums */
#define SNMP_SOCKETS    1  /* SNMP over Sockets, not lightweight API */
#define UDP_ECHO_SVR    1  /* built-in UDP Echo server */
#define DNS_CLIENT_UPDT 1  /* do Dynamic DNS Update */
#define TCP_WIN_SCALE   1  /* RFC1323 scaling TCP window feature */
#define TCP_TIMESTAMP   1  /* RFC1323 TCP timestamp feature */
#define NO_HTMLCOMP     1  /* Build WebServer without HTML Compiler */
#define USE_PPP         1  /* support PPP MAC iface(s) */
#define USE_MODEM       1  /* support PPP/SLIP on modems */
#define USE_COMPORT     1  /* use nvparms COMPORT numbering */
#define PPP_LOGFILE     1  /* ConPrintf() log to file option */
#define PAP_SUPPORT     1  /* Password authentication */
#define CHAP_SUPPORT    1  /* CHAP authentication */
#define PPP_DNS         1  /* domain nameserver address negotiation */
#define PPP_CHARIO      1  /* support  character read/write routines */
#define PPP_MENU        1  /* include the PPP debug menu */
#define PPP_VJC         1  /* VJ header compression */
#define USE_PPPOE       1  /* Support PPP over ethernet */
#define PPP_OLDFORMS    1  /* Use old (IP-Addresses) option */
#define PPP_PACKETS     1  /* support packet oriented read/write routines */
#define MSCHAP_SUPPORT  1  /* Microsoft CHAP authentication         */
#define LB_XOVER        1  /* cross 2 loopback lines (units 1 & 2) for test */
#define PPP_DHCP_CLIENT 1  /* get IP address via dhcp after IPCP is up */
#define PING_APP        1  /* application provides a pingUpcall */
#define IN_MENUS        1  /* support for InterNiche menu system */
#define VFS_FILES       1  /* Virtual File System */
#define DNS_CLIENT      1  /* include DNS client code */
#define INCLUDE_NVPARMS 1  /* include 'pretend' NVRAM stuff */
#define TELNET_SVR      1  /* TELNET server */
#define FTP_SERVER      1  /* FTP server */
#define FTP_CLIENT      1  /* include FTP client code */
#define TFTP_CLIENT     1  /* include TFTP client code */
#define TFTP_SERVER     1  /* include TFTP server code */
#define TK_STDIN_DEVICE      1  /* include keyboard task */
#define MAC_LOOPBACK    1  /* use MAC layer pseudo-Ethernet loopback driver */
#define DHCP_CLIENT     1  /* include DHCP client code */
#define IP_ROUTING      1  /* do real IP routing, not imbedded agent fake route */
#define INCLUDE_TCP     1  /* this link will include NetPort TCP w/MIB */
#define TCP_ZEROCOPY    1  /* Use TCP Zero Copy */
#define IP_FRAGMENTS    1  /* IP fragment reassembly */
#define NET_STATS       1  /* include statistics printfs */
#endif

/**** end of option list ***/


/* default setups of some sub-options */


#define UCOS_II         1  /* Build with uCOS-II RTOS (ver 2.76) */
#define ALTERA_NIOS2    1  /* Build for the Altera NIOS 2 Cyclone Dev Board */

#define NO_INICHE_EXTENSIONS   1   /* NO Interniche Extensions to MicroC/OS-II */

#ifdef UCOS_II
#define CHRONOS         1  /* Define CHRONOS Macro to hook into uCOS-II RTOS */
#define BLOCKING_APPS   1  /* applications block rather than poll */
#endif

#ifdef ALTERA_NIOS2
#define _SDS_CC_         1
#endif

/* if any SNMP agent is used, then INCLUDE_SNMP should be enabled */
#if (defined(INCLUDE_SNMPV1)||defined(INCLUDE_SNMPV2C)||defined(INCLUDE_SNMPV3))
#define INCLUDE_SNMP    1  /* update SNMP counters in TCPIP stack */
#endif

#ifdef INCLUDE_SNMP
#if !defined(PREBIND_AGENT) && !defined(SNMP_SOCKETS)
#define PREBIND_AGENT   1   /* hardcode SNMP port into UDP */
#endif
#endif

#ifdef LB_XOVER
#ifndef MULTI_HOMED
#define MULTI_HOMED 1
#endif
#endif

#ifdef MULTI_HOMED
#ifndef IP_ROUTING
#define IP_ROUTING 1    /* force IP routing on multi-homed compiles */
#endif
#endif

#ifdef INCLUDE_TCP
#ifndef FULL_ICMP
#define FULL_ICMP 1     /* force full ICMP for TCP support */
#endif
#endif

#ifdef MULTI_HOMED
#define STATIC_NETS   4  /* static network interfaces to allow for... */  
#else
#define STATIC_NETS   1
#endif

#ifdef DYNAMIC_IFACES
#define MAXNETS       8  /* max ifaces to support at one time */
#else
#define MAXNETS       STATIC_NETS
#endif

#ifdef DYNAMIC_IFACES
#ifndef MULTI_HOMED
#error MULTI_HOMED needs to be defined if using Dynamic Ifaces...
#endif
#endif   /* DYNAMIC_IFACES */

#ifdef IP_FRAGMENTS
#ifndef MAX_FRAG_PKTS
#define MAX_FRAG_PKTS 5 /* max IP pkts we can simultaneously reassemble */
#endif
#endif

#ifdef  DNS_CLIENT          
#define MAXDNSSERVERS 3  /* # of nameservers the DNS client can use */
#endif

/*
 * VFS options
 */
#ifdef VFS_FILES
#define UNIX_VFS        1  /* make VFS look UNIX-like */
#define HT_RWVFS        1  /* support read/write VFS */
#define HT_EXTDEV       1  /* allow custom file devices (memdev, null, etc.) */
#define VFS_STRIPPATH   1  /* used to strip path for filenames in vfsfiles.c */

#ifdef NOTDEF   /* unused VFS options */
#define HT_LOCALFS      1  /* layer VFS on target system's local file system */
#define HT_SYNCDEV      1  /* support sync to flash device */
#define VFSLOCK_TX_SEMAPHORE  1
#endif

#if defined(VFS_FILES) && defined(VFSLOCK_TX_SEMAPHORE)
extern void vfs_lock(void);
extern void vfs_unlock(void);
#else
#define vfs_lock()      /* define pesky macros away */
#define vfs_unlock()
#endif
#endif /* VFS_FILES */


/* 
 * VFS memory/null device options
 *
 * set up the memory device for the Integrator board: 512KB at 0 
 */
#ifdef VFS_FILES
#define MEMDEV_BASE 0x00200000
#define MEMDEV_SIZE 0x00100000
#endif /* VFS_FILES */

#ifdef C_CHECKSUM
#define cksum ccksum
#endif  /* C_CHECKSUM */

/* Sanity Check for OS Type */

#if defined(INICHE_TASKS) && defined(UCOS_II)
#error Only one of "INICHE_TASKS" and "UCOS_II" should be defined.
#endif

#ifndef SUPERLOOP
#define OSPORT_H "osport.h"
#endif /* SUPERLOOP */

/* 
 * Task/Thread synchronization primitives
 * 
 * The stack supports two methods of protecting its queues and data
 * structures from destructive re-entry by ISRs, multiple tasks, etc. One
 * is to lock a "resource ID" which makes code wait when trying to access
 * a shared resource, the other is a CRITICAL SECTION frame. You should
 * use one or the other, not both. See the porting guide for details.
 *
 * For this ipport.h_h, we support building to run with NicheTask and
 * critical section, ThreadX, PSOS and VxWorks with resource locking.
 *
 * Hence the use of the THREADX, VX_WORKS or PSOS macro to select one or 
 * the other.
 */

#if !defined(UCOS_II)

/*
 * Use the critical section routines, and map ENTER/EXIT_CRIT_SECTION
 * to them.  Also use macros to provide null definitions of 
 * the LOCK/UNLOCK_NET_RESOURCE routines so they become no-ops in the
 * code.
 *
 * Enter and Exit critical sections keep track of a 'nesting' level.
 */

extern void enter_critical_section(void *);
extern void exit_critical_section(void *);
#define ENTER_CRIT_SECTION(p)  enter_critical_section(p)
#define EXIT_CRIT_SECTION(p)   exit_critical_section(p)

#define LOCK_NET_RESOURCE(res)
#define UNLOCK_NET_RESOURCE(res)

/* #define CRIT_DEBUG 1 */

#else /* UCOS_II so USE_RESOURCE_LOCKS */

/* 
 * Use the resource locking routines, and map LOCK/UNLOCK_NET_RESOURCE 
 * to them with macros.  Similar to the above case, we use macros to 
 * provide null definitions for ENTER/EXIT_CRITICAL_SECTION.
 */

extern void LOCK_NET_RESOURCE(int res);
extern void UNLOCK_NET_RESOURCE(int res);

/* 
 * also define macros for these so that tk_psos.h or tk_thrdx.h don't try 
 * to use their own prototypes 
 */
#define LOCK_NET_RESOURCE(x)    LOCK_NET_RESOURCE(x)
#define UNLOCK_NET_RESOURCE(x)  UNLOCK_NET_RESOURCE(x)

/* #define Integrator port resource IDs as indexes into net-resources[] array */
#define NET_RESID   0    /* stack code resource lock */
#define RXQ_RESID   1    /* received packet queue resource lock */
#define FREEQ_RESID 2    /* free packet queue resource lock */

/* ...and macro-map critical-section calls to no-ops */
#define ENTER_CRIT_SECTION(p)
#define EXIT_CRIT_SECTION(p)

#endif /* UCOS_II */

/*
 * tcp_sleep()/tcp_wakeup() definitions
 */
#ifdef INICHE_TASKS     /* InterNiche multitasking system */
#define TCPWAKE_ALREADY 1  /* use per-port sleep, wakeup */
#define tcp_sleep(ev)   tk_ev_block(ev)
#define tcp_wakeup(ev)  tk_ev_wake(ev)
/* ISI PSOS or Express Logic ThreadX RTOS */
#elif defined(UCOS_II)
#define TCPWAKE_ALREADY 1  /* use per-port sleep, wakeup */

#ifdef NOT_DEF
#endif /* NOT_DEF */

void tcp_sleep(void * event);
void tcp_wakeup(void * event);

#endif

/*
 * include tasking package definitions
 */
#ifdef INICHE_TASKS     /* InterNiche multitasking system */
#include "tk_ntask.h"
#elif defined(UCOS_II)  /* Interniche uCOS-II RTOS */
#include "tk_crnos.h"
#endif

/* #ticks per 0.1 s; used by misclib/netmain.c to drive inet_timer()
 * If, however, you need to change the #times to invoke inet_timer(),
 * change this value
 */
#define SYS_SHORT_SLEEP      1

#define YIELD tk_yield

#ifdef SUPERLOOP
extern void tk_yield(void);
#endif

/* number of entries in IP routing table */
#define RT_TABS   16

/* override ETHHDR_SIZE to introduce padding so that ARP, IP headers
 * will be aligned on a four-byte boundary
 */
#define ETHHDR_SIZE     16


/* Since ETHHDR_SIZE is 16, received ethernet packets start 2 bytes into 
 * buffer 
 */
#define ETHHDR_BIAS  (2)


/* InterNiche stack generic error codes: generally full success is 0;
 * definite errors are negative numbers, and indeterminate conditions
 * are positive numbers. These may be changed if they conflict with
 * defines in the target system. They apply to the NetPort IP stack,
 * the IPX logic, and the portable SNMP Agent. If you have to change
 * these values, be sure to recompile ALL NetPort sources.
 */

#define SUCCESS            0   /* whatever the call was, it worked. */

/* programming errors */
#define ENP_PARAM        -10  /* bad parameter */
#define ENP_LOGIC        -11  /* sequence of events that shouldn't happen */

/* system errors */
#define ENP_NOMEM        -20  /* malloc or calloc failed */
#define ENP_NOBUFFER     -21  /* ran out of free packets */
#define ENP_RESOURCE     -22  /* ran out of other queue-able resource */
#define SEND_DROPPED     ENP_RESOURCE   /* full queue or similar lack of resource */
#define ENP_BAD_STATE    -23  /* TCP layer error */
#define ENP_TIMEOUT      -24  /* TCP layer error */
#define ENP_NOFILE       -25  /* expected file was missing */
#define ENP_FILEIO       -26  /* file IO error */

/* net errors */
#define ENP_SENDERR      -30  /* send to net failed at low layer */
#define ENP_NOARPREP     -31  /* no ARP for a given host */
#define ENP_BAD_HEADER   -32  /* bad header at upper layer (for upcalls) */
#define ENP_NO_ROUTE     -33  /* can't find a reasonable next IP hop */
#define ENP_NO_IFACE     -34  /* can't find a reasonable interface */
#define ENP_HARDWARE     -35  /* detected hardware failure */

/* conditions that are not really fatal OR success: */
#define ENP_SEND_PENDING   1  /* packet queued pending an ARP reply */
#define ENP_NOT_MINE       2  /* packet was not of interest (upcall reply) */
#define ARP_WAITING      ENP_SEND_PENDING /* ARP holding pkt, waiting for response from fhost */

/* A portable macro to check whether a ptr is within a certain range.
 * In an environment where selectors are important, this macro will
 * be altered to reflect that.
 */
 
#define IN_RANGE(p1, len1, p2) \
   (((p1) <= (p2)) && (((p1) + (len1)) > (p2)))
 
/* define the various IP stack block and buffer allocation routines */

char * npalloc(unsigned size);
void   npfree(void * ptr);

#define RT_ALLOC(size)     npalloc(size)  /* route block alloc */
#define RT_FREE(ptr)       npfree(ptr)

/*
 * The triple speed mac driver utilizes a scatter-gather DMA
 * controller to transfer data between Ethernet MAC and Nios
 * system memory. For Nios systems with data cache, all packet
 * data structures must therfore be marked as uncached. The "nc"
 * palloc/pfree routines wrap the default npalloc()/npfree()
 * with Nios II library calls to mark the returned pointers as
 * cache-bypassed.
 */
#ifdef ALTERA_TRIPLE_SPEED_MAC
char * ncpalloc(unsigned size);
void ncpfree(void *ptr);

#define BB_ALLOC(size)     ncpalloc(size)  /* Big packet buffer alloc */
#define BB_FREE(ptr)       ncpfree(ptr)
#define LB_ALLOC(size)     ncpalloc(size)  /* Little packet buffer alloc */
#define LB_FREE(ptr)       ncpfree(ptr)
#else
#define BB_ALLOC(size)     npalloc(size)  /* Big packet buffer alloc */
#define BB_FREE(ptr)       npfree(ptr)
#define LB_ALLOC(size)     npalloc(size)  /* Little packet buffer alloc */
#define LB_FREE(ptr)       npfree(ptr)
#endif

#define NB_ALLOC(size)     npalloc(size)  /* netbuf structure alloc */
#define NB_FREE(ptr)       npfree(ptr)
#define UC_ALLOC(size)     npalloc(size)  /* UDP connection block alloc */
#define UC_FREE(ptr)       npfree(ptr)
#define TK_ALLOC(size)     npalloc(size)  /* task control block */
#define TK_FREE(ptr)       npfree(ptr)
#define STK_ALLOC(size)    npalloc(size)  /* task stack */
#define STK_FREE(ptr)      npfree(ptr)
#define PMTU_ALLOC(size)   npalloc(size)  /* path MTU cache */
#define PMTU_FREE(ptr)     npfree(ptr)

#ifdef DYNAMIC_IFACES
#define NET_ALLOC(size)    npalloc(size)  /* struct net */
#define NET_FREE(ptr)      npfree(ptr)
#define IFM_ALLOC(size)    npalloc(size)  /* struct IfMib */
#define IFM_FREE(ptr)      npfree(ptr)
#endif   /* DYNAMIC_IFACES */

/* map memory routines to standard lib */
#define MEMCPY(dst,src,size)  memcpy((void *)(dst),(void *)(src),(size))
#define MEMMOVE(dst,src,size) memmove((void *)(dst),(void *)(src),(size))
#define MEMSET(ptr,val,size)  memset((void *)(ptr),(val),(size))
#define MEMCMP(p1,p2,size)    memcmp((void *)(p1),(void *)(p2),(size))

extern void dtrap( void );

#ifdef STRUCTURE_PACKING_OK
/*
 * Structure packing is achieved using the __packed directive
 */
#define START_PACKED_STRUCT(sname)    __packed struct sname {
#define END_PACKED_STRUCT(sname)      };
#else
/*
 * Toolchain/target structure-packing definitions
 *
 * NO_CC_PACKING - define this to indicate that the toolchain and/or
 *   target don't support packed structures
 * START_PACKET_STRUCT()/END_PACKED_STRUCT() - define these with
 *   the appropriate start/finish of a packed structure declaration
 */
#define NO_CC_PACKING    1   /* CPU/Compiler combo can't pack structs */
/* ...thus the structs we'd normally define packed are normal */
#define START_PACKED_STRUCT(sname)  struct sname {
#define END_PACKED_STRUCT(sname)  };
#endif /* STRUCTURE_PACKING_OK */

/* port routine/macro to wake packet demuxer task after receives have
 * been placed in rcvdq by drivers. Define to NULL for Superloop demo:
 */
#ifdef  SUPERLOOP
#define  SignalPktDemux()
#endif  /* SUPERLOOP */

/* UART driver header */
#define UART_INCLUDE "../nios2gcc/uart.h"

#ifdef NATIVE_PRINTF
/*
 * Use native printf from the C library for debug output.
 */
#define dprintf  printf
#define dputchar putchar
#else
/*
** Use UART driver for debug output
*/
#ifdef putchar
#undef putchar
#endif
#define putchar dputchar
#define printf  dprintf
#ifndef _IN_TTYIO_
void dprintf( char *, ... );
#endif
void dputchar( int );
#endif

/* Send startup errors to the right place */
#define initmsg  dprintf

/* net stats printf with redirectable device: */
extern int ns_printf( void * pio, char * format, ... );

#define COMMSIZE       64    /* max bytes in community string */

#define DELAY_REPLY     1    /* process incoming packets in background loop */

#define INETNET         0    /* index into nets[] for internet iface */
#define LOCALNET        1    /* index into nets[] for local net iface */

int   prep_ppp(int);    /* in ..\ppp\sys_np.c */
int   prep_ifaces(int firstIface);   /* set up interfaces */

/* 
 * define number and sizes of free packet buffers 
 */
#define NUMBIGBUFS   50
#define NUMLILBUFS   50

/* some maximum packet buffer numbers */
#define MAXBIGPKTS   50
#define MAXLILPKTS   50
#define MAXPACKETS (MAXLILPKTS+MAXBIGPKTS)

#define BIGBUFSIZE   1536
#define LILBUFSIZE   128

#define MAX_NVSTRING 128       /* MAX length of an nvparms string */

/* 
 * HUGE - define away this Intel-ism
 */
#define HUGE

/* Likewise with FAR */
#define FAR

/* macros to get rid of "unused argument" warnings. With compilers that
 * can suppress these warnings, define this to nothing.
 */
#define USE_ARG(x) (x++)
#define USE_VOID(x)

#undef ferror
#undef clearerr
#define CADDRT_ALREADY      1
#define SOCKERRORS_ALREADY  1

/* Almost */
#define     EHAVEOOB        217
#define     EIEIO           227 /* bad input/output on Old Macdonald's farm :-) */

#include <sys/errno.h>

#endif /* _IPPORT_H_ */

/* end of file ipport.h */
